#!/usr/bin/perl

my $curDir = $0;
if($curDir =~ m/\//) {
    $curDir =~ s/[^\/]*$//;
    chdir($curDir) or die $!;
}


use TParse;
use strict;


#key = "tid,syscall", value = issue time
my %entryTimes;


main();
sub main() {

    while(my $line = <STDIN>) {
	chomp($line);

	if($line =~ m/^\s*$/) {
	    #skip blank lines
	    next;
	}

	if($line =~ m/\*$/) {
	    print STDERR "Skipping comment:  $line\n";
	    #skip comments
	    next;
	}


	my @curArgs = TParse::parseLine($line);
	my $curCall = $curArgs[3];

	my $key = TParse::arg(\@curArgs, 'tid').','.TParse::arg(\@curArgs, 'syscall');

	if(TParse::arg(\@curArgs, 'probe_type') eq 'entry') {
	    if(defined $entryTimes{$key}) {
		die "Double entry for tid,syscall " . $key . " at time " . TParse::arg(\@curArgs, 'time') . "\n";
	    }
	    $entryTimes{$key} = TParse::arg(\@curArgs, 'time');
	} elsif(TParse::arg(\@curArgs, 'probe_type') eq 'return') {
	    if(! defined $entryTimes{$key}) {
		die "Found a return without an entry!";
	    }
	    my $entryTime = $entryTimes{$key};
	    delete $entryTimes{$key};

	    for (my $i = 0; $i <= $#curArgs; $i++) {
		if($i == TParse::argIndex($curCall,'time')) {
		    print "[$entryTime]; ";
		} elsif($i == TParse::argIndex($curCall,'probe_type')) {
		    print '[entry]; ';
		} else {
		    print "[$curArgs[$i]]; ";
		}
	    }
	    print "\n";

	    for (my $i = 0; $i <= $#curArgs; $i++) {
		print "[$curArgs[$i]]; ";
	    }
	    print "\n";
	} else {
	    print "$line\n";
	}
    }
}

