#!/usr/bin/perl

use strict;
use TParse;
use FilePaths;



my @lineBuffer;

#key = inode, val = path
my %inodeLookups;



sub getNextLine {
    if($#lineBuffer < 0) {
	if(my $line = <STDIN>) {
	    chomp($line);
	    push(@lineBuffer, $line);
	} else {
	    return '';
	}
    }

    return shift(@lineBuffer);
}



sub peekLine {
    $#_ == 0 or die "Bad args to peekLine!\n";
    my ($index) = @_;

    while(($#lineBuffer < $index) and (my $line = <STDIN>)) {
	chomp($line);
	push(@lineBuffer, $line);
    }

    if($#lineBuffer >= $index) {
	return $lineBuffer[$index];
    } else {
	return '';
    }
}



#this looks ahead at the next lines for an aio_return
#that will tell us the size of the file
sub getRetSize {
    $#_ == 0 or die "Bad args to getRetSize!\n";
    my ($aiocbp) = @_;
    my $index = 0;

    while(my $line = peekLine($index)) {
	my @args = TParse::parseLine($line);
	my $syscall = TParse::arg(\@args, 'syscall');
	if($syscall eq 'aio_return' and TParse::arg(\@args,'aiocbp') eq $aiocbp) {
	    return TParse::arg(\@args,'ret');
	} elsif (($syscall eq 'aio_read' or $syscall eq 'aio_write') and 
		   TParse::arg(\@args,'probe_type') eq 'entry' and 
		   TParse::arg(\@args,'aiocbp') eq $aiocbp) {

	    die "$syscall is reusing aiocbp $aiocbp!";
	}

	$index++;
    }

    die "Couldn't get return size for aiocbp $aiocbp";
}



main();
sub main {
    my $line;
    while($line = getNextLine()) {
	my @args = TParse::parseLine($line);
	my $syscall = TParse::arg(\@args, 'syscall');

	if($syscall eq 'aio_read' or $syscall eq 'aio_write') {
	    TParse::setArg(\@args, 'retsize', getRetSize(TParse::arg(\@args, 'aiocbp')));
	    print (TParse::args2str(\@args) . "\n");
	} else {
	    print "$line\n";
	}
    }
}



