#!/usr/bin/perl

use strict;
use TParse;
use FilePaths;



my @lineBuffer;

#key = inode, val = path
my %inodeLookups;



sub getNextLine {
    if($#lineBuffer < 0) {
	if(my $line = <STDIN>) {
	    chomp($line);
	    push(@lineBuffer, $line);
	} else {
	    return '';
	}
    }

    return shift(@lineBuffer);
}



sub peekLine {
    $#_ == 0 or die "Bad args to peekLine!\n";
    my ($index) = @_;

    while(($#lineBuffer < $index) and (my $line = <STDIN>)) {
	chomp($line);
	push(@lineBuffer, $line);
    }

    if($#lineBuffer >= $index) {
	return $lineBuffer[$index];
    } else {
	return '';
    }
}



#this looks ahead at the next lines for a build_path comment
#that will tell us what the path name is for the inode
sub resolve {
    $#_ == 1 or die "Bad args to resolve!\n";
    my ($inode, $tid) = @_;
    my $index = 0;

    while(my $line = peekLine($index)) {
	my @args = TParse::parseLine($line);
	my $syscall = TParse::arg(\@args, 'syscall');
	if($syscall eq 'build_path' and TParse::arg(\@args,'inode') eq $inode) {
	    return TParse::arg(\@args,'path');
	}

	if(TParse::arg(\@args, 'tid') eq $tid and TParse::arg(\@args, 'probe_type')) {
	    #we don't want to go past the return event of the 
	    #call that had the path we're trying to resolve
	    last;
	}

	$index++;
    }

    return '';
}



main();
sub main {
    my $line;
    while($line = getNextLine()) {
	my @args = TParse::parseLine($line);
	my $syscall = TParse::arg(\@args, 'syscall');
	my @argNames = TParse::getArgNames($syscall);

	foreach my $argName (@argNames) {
	    #this is an arg that could potentially be resolved
	    if($argName =~ m/path/i) {
		if(TParse::arg(\@args, $argName) =~ m/^\/\.vol\/(\d+)\/(\d+)/) {
		    my $inode = $2;

		    if(TParse::arg(\@args, 'probe_type') eq 'entry') {
			my $resolution = resolve($2, TParse::arg(\@args, 'tid'));
			if($resolution eq '') {
			    print STDERR "Couldn't resolve $inode in $line\n";
			}
			$resolution = $resolution;
			my $arg = TParse::arg(\@args, $argName);
			$arg =~ s/^\/\.vol\/(\d+)\/(\d+)/$resolution/;
			$arg = FilePaths::simplifyPath($arg);
			TParse::setArg(\@args, $argName, $arg);
			$inodeLookups{$inode} = $resolution;
		    } elsif(TParse::arg(\@args, 'probe_type') eq 'return') {
			my $arg = TParse::arg(\@args, $argName);
			my $resolution = $inodeLookups{$inode};
			$arg =~ s/^\/\.vol\/(\d+)\/(\d+)/$resolution/;
			$arg = FilePaths::simplifyPath($arg);
			TParse::setArg(\@args, $argName, $arg);
		    }
		}
	    }
	}

	print (TParse::args2str(\@args) . "\n");
    }
}



