#!/usr/bin/perl

my $curDir = $0;
if($curDir =~ m/\//) {
    $curDir =~ s/[^\/]*$//;
    chdir($curDir) or die $!;
}


use TParse;
use FilePaths;
use strict;


my $curCall;
my @curArgs;
my %dirList;
my %newDirList;


main();
sub main() {
    die if $#ARGV != 0;
    my $dirFile = shift(@ARGV);


    #what dirs do we already track?
    if(-f $dirFile) {
	open DIRFILE, "<$dirFile";
	while(<DIRFILE>) {
	    chomp($_);
	    $dirList{$_} = 1;
	}
	close DIRFILE;
    }


    #read trace to find new paths
    while(my $line = <STDIN>) {
	chomp($line);

	if($line =~ m/^\s*$/) {
	    next;
	}

	if($line =~ m/\*$/) {
	    #skip this, because it's a comment (it ends in *)
	    next;
	}

	if($line =~ m/DTrace Controller Says/) {
	    #skip this, because it's a comment from the dtrace controller
	    next;
	}

	$line = FilePaths::simplifyPath($line);

	addPath($line);
    }

    
    #print stats
    my $inputDirs = (keys %newDirList);
    my $existingDirs = (keys %dirList);
    my $newDirs = 0;
    my @newDirList;
    for(keys %newDirList) {
	if(! exists($dirList{$_})) {
	    $newDirs++;
	    $dirList{$_} = 1;
	    push(@newDirList, $_);
	}
    }

    print "There were $existingDirs dirs being tracked.\n";
    print "Attempting to add $inputDirs dirs to tracking.\n";
    print "Tracking $newDirs new dirs.\n";
    if($newDirs <= 8) {
	print join("\n", @newDirList) . "\n";
    }
    print "There are now " . (keys %dirList) . " dirs being tracked.\n";

    #write updated list to file
    open DIRFILE, ">$dirFile" or die $!;
    foreach my $path (sort {$a<=>$b} keys %dirList) {
	print DIRFILE $path . "\n";
    }
    close DIRFILE;
}



sub addPath {
    die if $#_ != 0;
    my $path = shift(@_);

    if(-d $path) {
	$newDirList{$path} = 1;
    } else {
	$path =~ s/\/[^\/]*$//;
	$newDirList{$path} = 1;
    }
}

