#!/usr/bin/perl

#output lines like this to init_symlinks.txt:
# ["symlink"]; [path]; [target]; 
# OR
# ["file"]; [inode]; [path]; [size]; 


use strict;
use TParse;
use FilePaths;
use File::Basename;

#key = path, val = target
my %symlinks;
#key = inode, val = size
my %inode2size;
#key = inode, val = path
my %inode2path;
#directories
my %dirs;


main();
sub main {
    #iterate over each dir in the file
    while(my $dirName = <STDIN>) {
	chomp($dirName);
	my $target;

	if(opendir(DIR, $dirName)) {
	    while(my $fileName = readdir(DIR)) {
		my $path = "$dirName/$fileName";
		checkPath($path);
	    }
	    closedir DIR;
	}
    }

    foreach my $key (sort{$a cmp $b} keys %dirs) {
	print("[dir]; [$key]; \n");
    }

    foreach my $key (sort{$a cmp $b} keys %inode2size) {
	print("[file]; [$key]; [$inode2path{$key}]; [$inode2size{$key}]; \n");
    }

    foreach my $key (sort{$a cmp $b} keys %symlinks) {
	print("[symlink]; [$key]; [$symlinks{$key}]; \n");
    }
}


sub checkPath {
    $#_ == 0 or die "Bad arguments to checkPath";
    my ($path) = @_;

    if($path =~ /^\/dev/) {
	return;
    }

    my @parts = split('/', $path);
    shift(@parts); #the first element is empty

    #if $path is /a/b/c/d, loop over /a, /a/b, /a/b/c, and a/b/c/d
    for (my $i = 0; $i <= $#parts; $i++) {
	my $dir = ('/' . join('/', @parts[0..$i]));
	my $target = getAbsSymlink($dir);

	if($target ne '') {
	    #we found a symlink
	    addSymlink($dir, $target);
	    $dir = quotemeta($dir);
	    $path =~ s/^$dir/$target/;
	    checkPath($path); #check the resolved path
	    return; #stop checking with the path that contains the symlink
	} elsif ($i < $#parts) {
	    #this is a directory we can add to our hash set
	    $dirs{$dir} = 1;
	}
    }

    #if we get to this point, the path is real all the way (no symlinks)
    if(-f $path) {
	my @results;

	@results = stat($path);
	die if $#results == -1;
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)
	    = @results;
	$path = FilePaths::simplifyPath($path);
	$inode2size{$ino} = $size;
	$inode2path{$ino} = $path;

	#also get the rsrc fork
	$path .= '/..namedfork/rsrc';

	@results = stat($path);
	if($#results != -1) {
	    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)
		= @results;
	    $ino .= 'rsrc';
	    $path = FilePaths::simplifyPath($path);
	    $inode2size{$ino} = $size;
	    $inode2path{$ino} = $path;
	}
    }
}


sub addSymlink {
    $#_ == 1 or die "Bad arguments to printSymLink";
    my ($path, $target) = @_;
    $path = FilePaths::simplifyPath($path);
    $target = FilePaths::simplifyPath($target);

    if(exists $symlinks{$path}) {
	$symlinks{$path} eq $target or die "Died in addSymlink: '$target' ne '$symlinks{$path}'\n";
    } else {
	$symlinks{$path} = $target;
    }
}




sub getAbsSymlink {
    $#_ == 0 or die "Bad arguments to getAbsSymlink";
    my ($path) = @_;
    my $target = readlink($path);

    if($target eq '') {
	return '';
    }

    if($target !~ m/^\//) {
	#this is relative, so make it absolute
	$target = dirname($path) . '/' . $target;
	$target = FilePaths::simplifyPath($target);
    }

    return $target;
}


