#!/usr/sbin/dtrace -C  -s
#pragma D option quiet
#pragma D option dynvarsize=2M
#pragma D option specsize=1M
#pragma D option bufsize=110M
#pragma D option switchrate=50hz
#pragma D option destructive

#include <sys/mman.h>
#include <aio.h>
#include <fcntl.h>

#define WATCHCOND (execname==progName)

#define PRINT_GENERIC_ENTRY  printf("[%d]; [%d]; [entry]; [%s]; ",  timestamp - timeZero, tid, probefunc);
#define PRINT_GENERIC_RETURN printf("[%d]; [%d]; [return]; [%s]; ", timestamp - timeZero, tid, probefunc);
#define PRINT_GENERIC_COMMENT printf("[%d]; [N/A]; [comment]; ", timestamp - timeZero);

enum aio_rw { AIO_READ, AIO_WRITE };



dtrace:::BEGIN
{
	/* The name of the program we're supposed to trace */
	progName = $1;


	timeZero = timestamp; /* we will use this to compute time elapsed since trace started running (not the program). */

	/* the third argument to lseek (whence) specifies what the offset is relative to */
	lseek_whenceNames[0] = "SEEK_SET";
	lseek_whenceNames[1] = "SEEK_CUR";
	lseek_whenceNames[2] = "SEEK_END";

        fcntl_commandNames[0] = "F_DUPFD";
        fcntl_commandNames[1] = "F_GETFD";
        fcntl_commandNames[2] = "F_SETFD";
        fcntl_commandNames[3] = "F_GETFL";
        fcntl_commandNames[4] = "F_SETFL";
        fcntl_commandNames[5] = "F_GETOWN";
        fcntl_commandNames[6] = "F_SETOWN";
        fcntl_commandNames[7] = "F_GETLK";
        fcntl_commandNames[8] = "F_SETLK";
        fcntl_commandNames[9] = "F_SETLKW";
        fcntl_commandNames[41] = "F_CHKCLEAN";
        fcntl_commandNames[42] = "F_PREALLOCATE";
        fcntl_commandNames[43] = "F_SETSIZE";
        fcntl_commandNames[44] = "F_RDADVISE";
        fcntl_commandNames[45] = "F_RDAHEAD";
        fcntl_commandNames[46] = "F_READBOOTSTRAP";
        fcntl_commandNames[47] = "F_WRITEBOOTSTRAP";
        fcntl_commandNames[48] = "F_NOCACHE";
        fcntl_commandNames[49] = "F_LOG2PHYS";
        fcntl_commandNames[50] = "F_GETPATH";
        fcntl_commandNames[51] = "F_FULLFSYNC";
        fcntl_commandNames[52] = "F_PATHPKG_CHECK";
        fcntl_commandNames[53] = "F_FREEZE_FS";
        fcntl_commandNames[54] = "F_THAW_FS";
        fcntl_commandNames[55] = "F_GLOBAL_NOCACHE";
        fcntl_commandNames[56] = "F_OPENFROM";
        fcntl_commandNames[57] = "F_UNLINKFROM";
        fcntl_commandNames[58] = "F_CHECK_OPENEVT";
        fcntl_commandNames[59] = "F_ADDSIGS";
        fcntl_commandNames[60] = "F_MARKDEPENDENCY";

	socket_domainNames[1] = "AF_UNIX";
	socket_domainNames[2] = "AF_INET";
	socket_domainNames[7] = "AF_ISO";
	socket_domainNames[6] = "AF_NS";
	socket_domainNames[3] = "AF_IMPLINK";
	socket_domainNames[17] = "AF_ROUTE";
}


dtrace:::BEGIN
{
	PRINT_GENERIC_COMMENT
	printf("[setter]; [cwd]; [%s]; \n", cwd);
}




syscall::write:entry,
syscall::write_nocancel:entry
/ arg0 == 4321 /
{
	/* This is just so we can print arbitrary messages to the buffer. */
	/* The dtracecontroller program does writes to fd 4321 that we can grab */
	/* what it says.  We assume fd 4321 will never be used for anything else :) */

	PRINT_GENERIC_COMMENT
	printf("[controller]; [%s]; \n", copyinstr(arg1));
}

syscall::write:entry,
syscall::write_nocancel:entry
/ arg0 == 4321 && copyinstr(arg1) == "quit" /
{
	/* this is using the same dtracecontroller trick used in the above handler */
	exit(0);
}





/*We can use this to get a list of every syscall called by the app*/

syscall:::entry
/ WATCHCOND /
{
	@syscalls[probefunc] = count();
}





proc:::exit
/WATCHCOND/
{
	exit(0);
}



syscall::open:return,
syscall::open_nocancel:return,
syscall::open_extended:return
/ WATCHCOND /
{
	self->open_type =  -1;
	self->open_inode =  -1;
	self->open_parent_inode =  -1;
	self->open_linkcount =  -1;
	self->open_desc_inode =  -1;
}



syscall::open:return,
syscall::open_nocancel:return,
syscall::open_extended:return
/ WATCHCOND && arg0 != -1 && 
  ((struct proc *) curpsinfo)->p_fd->fd_ofiles[arg1]->f_fglob->fg_type == DTYPE_VNODE
/
{	
	this->vnode_ptr =  (struct vnode *)(((struct proc *)curpsinfo)->p_fd->fd_ofiles[arg1]->f_fglob->fg_data) ;
	this->cnode_ptr = (struct cnode *)(this->vnode_ptr->v_data);

	self->open_type = this->vnode_ptr->v_type;
	self->open_inode = (this->cnode_ptr->c_attr).ca_fileid;
	self->open_parent_inode = (this->cnode_ptr->c_desc).cd_parentcnid;

	self->open_linkcount = (this->cnode_ptr->c_attr).ca_linkcount;
	self->open_desc_inode = (this->cnode_ptr->c_desc).cd_cnid;

	self->open_offset = ((struct proc *)curpsinfo)->p_fd->fd_ofiles[arg1]->f_fglob->fg_offset;
}



syscall::read:entry,
syscall::read_nocancel:entry
/ WATCHCOND /
{
	/* this isn't really necessary, but DTrace complains it's not 
	  initialized otherwise */
	self->read_fildes = NULL; 
}




syscall::write:entry,
syscall::write_nocancel:entry
/ WATCHCOND /
{
	/* this isn't really necessary, but DTrace complains it's not 
	  initialized otherwise */
	self->write_fildes = NULL; 
}



syscall::read:return,
syscall::read_nocancel:return
/ WATCHCOND /
{
	self->read_offset =  -1;
}


syscall::read:return,
syscall::read_nocancel:return
/ WATCHCOND && arg0 != -1 && 
  ((struct proc *) curpsinfo)->p_fd->fd_ofiles[self->read_fildes]->f_fglob->fg_type == DTYPE_VNODE
/
{	
	self->read_offset = ((struct proc *)curpsinfo)->p_fd->fd_ofiles[self->read_fildes]->f_fglob->fg_offset;
}




syscall::write:return,
syscall::write_nocancel:return
/ WATCHCOND /
{
	self->write_offset =  -1;
}


syscall::write:return,
syscall::write_nocancel:return
/ WATCHCOND && arg0 != -1 && 
  ((struct proc *) curpsinfo)->p_fd->fd_ofiles[self->write_fildes]->f_fglob->fg_type == DTYPE_VNODE
/
{	
	self->write_offset = ((struct proc *)curpsinfo)->p_fd->fd_ofiles[self->write_fildes]->f_fglob->fg_offset;
}



syscall::readlink:entry
/ WATCHCOND /
{
	/* this isn't really necessary, but DTrace complains it's not 
	  initialized otherwise */
	self->readlink_buf = NULL; 
}

syscall::readlink:return
/ WATCHCOND /
{
	self->readlink_target = "";
}

syscall::readlink:return
/ WATCHCOND && arg0 != -1 /
{
	this->target_arr = (char *)copyin((user_addr_t)self->readlink_buf, arg0+1);
	this->target_arr[arg0] = 0;
	self->readlink_target = stringof(this->target_arr);
}









fbt::build_path:entry
/ WATCHCOND /
{
	self->build_path_vnode_ptr = (struct vnode *)arg0;
	self->build_path_buff = (char *)arg1;
}

fbt::build_path:return
/ WATCHCOND /
{
	PRINT_GENERIC_COMMENT
	this->cnode_ptr = (struct cnode *)(self->build_path_vnode_ptr->v_data);
	this->inode = (this->cnode_ptr->c_attr).ca_fileid;
	printf("[build_path]; [%d]; [%s]; \n", this->inode, stringof(self->build_path_buff));
}


fbt::VNOP_EXCHANGE:entry
/ WATCHCOND /
{
	self->exchangedata_vnode1 = (struct vnode *)arg0;
	self->exchangedata_vnode2 = (struct vnode *)arg1;
}

fbt::hfs_vnop_pagein:entry
/ WATCHCOND /
{
	self->pagein_args = (struct vnop_pagein_args *)arg0;
	PRINT_GENERIC_ENTRY
	printf("\n");
}    

fbt::hfs_vnop_pagein:return
/ WATCHCOND /
{
	PRINT_GENERIC_RETURN
	this->cnode_ptr = (struct cnode *)(self->pagein_args->a_vp->v_data);
	this->inode = (this->cnode_ptr->c_attr).ca_fileid;
	/*Copied from hfs_vnop_pagein in hfs_readwrite.c ; uses size of file if offset input is greater than actual size.*/
	this->file_size = ((self->pagein_args->a_vp) == this->cnode_ptr->c_rsrc_vp ? this->cnode_ptr->c_rsrcfork->ff_data.cf_size : this->cnode_ptr->c_datafork->ff_data.cf_size);
	this->actual_size = ((this->file_size < self->pagein_args->a_size)? this-> file_size : self->pagein_args->a_size);
	printf("[%d]; [%d]; [%d]; [%s]; [%d]; [%d]; \n", self->pagein_args->a_f_offset, self->pagein_args->a_size, this->inode, stringof(self->pagein_args->a_vp->v_name), this->actual_size,
	    arg0);
} 



syscall::rename:entry
/ WATCHCOND /
{
	self->rename_inode = -1;
	self->rename_type  = -1;
}

fbt::VNOP_RENAME:entry
/ WATCHCOND /
{
	self->vnop_rename_fvp = (struct vnode *)arg1;
	self->vnop_rename_fdvp = (struct vnode *)arg0;
	self->vnop_rename_tdvp = (struct vnode *)arg3;
}

fbt::VNOP_RENAME:return
/ WATCHCOND /
{
	this->cnode_ptr = (struct cnode *)(self->vnop_rename_fvp->v_data);
	self->rename_inode = (this->cnode_ptr->c_attr).ca_fileid;
	self->rename_type = self->vnop_rename_fvp->v_type;

	this->cnode_from_dir_ptr = (struct cnode *)(self->vnop_rename_fdvp->v_data);
	this->cnode_to_dir_ptr = (struct cnode *)(self->vnop_rename_tdvp->v_data);
	self->rename_from_dir_inode = (this->cnode_from_dir_ptr->c_attr).ca_fileid;
	self->rename_to_dir_inode = (this->cnode_to_dir_ptr->c_attr).ca_fileid;	
}




dtrace:::END {
    PRINT_GENERIC_COMMENT;
    printf("[syscall_map]; [\n");
    printa(" %s %@d \n", @syscalls);
    printf("]; \n");
}



syscall::open:entry,
syscall::open_nocancel:entry,
syscall::open_extended:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->open_path = arg0;
	self->open_oflag = arg1;
	printf("\n");
} 

syscall::open:return,
syscall::open_nocancel:return,
syscall::open_extended:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->open_path));
	printf("[%d]; ", self->open_oflag);
	this->flags = "";
	this->flags = (!(self->open_oflag & O_WRONLY) && !(self->open_oflag & O_RDWR)) ? strjoin(this->flags, "O_RDONLY ") : this->flags;
	this->flags = (self->open_oflag & O_WRONLY) ? strjoin(this->flags, "O_WRONLY ") : this->flags;
	this->flags = (self->open_oflag & O_RDWR) ? strjoin(this->flags, "O_RDWR ") : this->flags;
	this->flags = (self->open_oflag & O_NONBLOCK) ? strjoin(this->flags, "O_NONBLOCK ") : this->flags;
	this->flags = (self->open_oflag & O_APPEND) ? strjoin(this->flags, "O_APPEND ") : this->flags;
	this->flags = (self->open_oflag & O_CREAT) ? strjoin(this->flags, "O_CREAT ") : this->flags;
	this->flags = (self->open_oflag & O_TRUNC) ? strjoin(this->flags, "O_TRUNC ") : this->flags;
	this->flags = (self->open_oflag & O_EXCL) ? strjoin(this->flags, "O_EXCL ") : this->flags;
	this->flags = (self->open_oflag & O_SHLOCK) ? strjoin(this->flags, "O_SHLOCK ") : this->flags;
	this->flags = (self->open_oflag & O_EXLOCK) ? strjoin(this->flags, "O_EXLOCK ") : this->flags;
	this->flags = (self->open_oflag & O_NOFOLLOW) ? strjoin(this->flags, "O_NOFOLLOW ") : this->flags;
	this->flags = (self->open_oflag & O_SYMLINK) ? strjoin(this->flags, "O_SYMLINK ") : this->flags;
	this->flags = (self->open_oflag & O_EVTONLY) ? strjoin(this->flags, "O_EVTONLY ") : this->flags;

	this->fd_ofiles = (struct fileproc **)(((struct proc *) curpsinfo)->p_fd->fd_ofiles);
	this->file_table = (char *)alloca(6*sizeof(char));

	this->file_table[0] = this->fd_ofiles[0] != NULL ? 'X' : '_';
	this->file_table[1] = this->fd_ofiles[1] != NULL ? 'X' : '_';
	this->file_table[2] = this->fd_ofiles[2] != NULL ? 'X' : '_';
	this->file_table[3] = this->fd_ofiles[3] != NULL ? 'X' : '_';
	this->file_table[4] = this->fd_ofiles[4] != NULL ? 'X' : '_';
	this->file_table[5] = '\0';
	printf("[%s]; ", this->flags); 
	printf("[%d]; ", self->open_type); 
	printf("[%d]; ", self->open_inode); 
	printf("[%s]; ", stringof(this->file_table)); 
	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::shm_open:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->shm_open_name = arg0;
	self->shm_open_oflag = arg1;
	printf("\n");
} 

syscall::shm_open:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->shm_open_name));
	printf("[%d]; ", self->shm_open_oflag);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::close:entry,
syscall::close_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->close_fildes = arg0;
	printf("\n");
} 

syscall::close:return,
syscall::close_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->close_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::dup:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->dup_fildes = arg0;
	printf("\n");
} 

syscall::dup:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->dup_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::kqueue:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	printf("\n");
} 

syscall::kqueue:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::socket:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->socket_domain = arg0;
	self->socket_type = arg1;
	self->socket_protocol = arg2;
	printf("\n");
} 

syscall::socket:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->socket_domain);
	printf("[%d]; ", self->socket_type);
	printf("[%d]; ", self->socket_protocol);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::socketpair:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->socketpair_domain = arg0;
	self->socketpair_type = arg1;
	self->socketpair_protocol = arg2;
	self->socketpair_socket_vector = arg3;
	printf("\n");
} 

syscall::socketpair:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->socketpair_domain);
	printf("[%d]; ", self->socketpair_type);
	printf("[%d]; ", self->socketpair_protocol);
	this->newfds = (int *)copyin((user_addr_t)self->socketpair_socket_vector, 2*sizeof(int));
	printf("[%d]; ", *(this->newfds)); 
	printf("[%d]; ", *(this->newfds + 1)); 
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::read:entry,
syscall::read_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->read_fildes = arg0;
	self->read_buf = arg1;
	self->read_nbyte = arg2;
	printf("\n");
} 

syscall::read:return,
syscall::read_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->read_fildes);
	printf("[%d]; ", self->read_nbyte);

	printf("[%d]; ", self->read_offset); 
	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::write:entry,
syscall::write_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->write_fildes = arg0;
	self->write_buf = arg1;
	self->write_nbyte = arg2;
	printf("\n");
} 

syscall::write:return,
syscall::write_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->write_fildes);
	printf("[%d]; ", self->write_nbyte);

	printf("[%d]; ", self->write_offset); 
	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::pread:entry,
syscall::pread_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->pread_fildes = arg0;
	self->pread_buf = arg1;
	self->pread_nbyte = arg2;
	self->pread_offset = arg3;
	printf("\n");
} 

syscall::pread:return,
syscall::pread_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->pread_fildes);
	printf("[%d]; ", self->pread_nbyte);
	printf("[%d]; ", self->pread_offset);

	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::pwrite:entry,
syscall::pwrite_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->pwrite_fildes = arg0;
	self->pwrite_buf = arg1;
	self->pwrite_nbyte = arg2;
	self->pwrite_offset = arg3;
	printf("\n");
} 

syscall::pwrite:return,
syscall::pwrite_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->pwrite_fildes);
	printf("[%d]; ", self->pwrite_nbyte);
	printf("[%d]; ", self->pwrite_offset);

	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::aio_read:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->aio_read_aiocbp = arg0;
	printf("\n");
} 

syscall::aio_read:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[0x%X]; ", self->aio_read_aiocbp);
	this->aiocb = (struct aiocb *)copyin((user_addr_t)self->aio_read_aiocbp, sizeof(struct aiocb));
	printf("[%d]; ", this->aiocb->aio_fildes); 
	printf("[%d]; ", curpsinfo->pr_dmodel == 1 ? *(long long int *)copyin((user_addr_t)(self->aio_read_aiocbp+4), sizeof(long long int)) : *(long long int *)copyin((user_addr_t)(self->aio_read_aiocbp+8), sizeof(long long int))); 
	printf("[%d]; ", curpsinfo->pr_dmodel == 1 ? *(long long int *)copyin((user_addr_t)(self->aio_read_aiocbp+16), sizeof(long long int)) : *(long long int *)copyin((user_addr_t)(self->aio_read_aiocbp+24), sizeof(long long int))); 
	printf("[%d]; ", 0); 
	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::aio_error:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->aio_error_aiocbp = arg0;
	printf("\n");
} 

syscall::aio_error:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[0x%X]; ", self->aio_error_aiocbp);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::aio_return:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->aio_return_aiocbp = arg0;
	printf("\n");
} 

syscall::aio_return:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[0x%X]; ", self->aio_return_aiocbp);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::aio_suspend:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->aio_suspend_list = arg0;
	self->aio_suspend_nent = arg1;
	self->aio_suspend_timeout = arg2;
	printf("\n");
} 

syscall::aio_suspend:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->aio_suspend_nent);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::lseek:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->lseek_fildes = arg0;
	self->lseek_offset = arg1;
	self->lseek_whence = arg2;
	printf("\n");
} 

syscall::lseek:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->lseek_fildes);
	printf("[%d]; ", self->lseek_offset);
	printf("[%d]; ", self->lseek_whence);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fsync:entry,
syscall::fsync_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fsync_fildes = arg0;
	printf("\n");
} 

syscall::fsync:return,
syscall::fsync_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fsync_fildes);

	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fcntl:entry,
syscall::fcntl_nocancel:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fcntl_fildes = arg0;
	self->fcntl_cmd = arg1;
	self->fcntl_arg = arg2;
	printf("\n");
} 

syscall::fcntl:return,
syscall::fcntl_nocancel:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fcntl_fildes);
	printf("[%d]; ", self->fcntl_cmd);
	printf("[%d]; ", self->fcntl_arg);
	
	printf("[%s]; ", fcntl_commandNames[self->fcntl_cmd]); 
	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::ftruncate:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->ftruncate_fildes = arg0;
	self->ftruncate_length = arg1;
	printf("\n");
} 

syscall::ftruncate:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->ftruncate_fildes);
	printf("[%d]; ", self->ftruncate_length);

	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::link:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->link_path1 = arg0;
	self->link_path2 = arg1;
	printf("\n");
} 

syscall::link:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->link_path1));
	printf("[%s]; ", copyinstr(self->link_path2));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::symlink:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->symlink_path1 = arg0;
	self->symlink_path2 = arg1;
	printf("\n");
} 

syscall::symlink:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->symlink_path1));
	printf("[%s]; ", copyinstr(self->symlink_path2));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::rmdir:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->rmdir_path = arg0;
	printf("\n");
} 

syscall::rmdir:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->rmdir_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::unlink:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->unlink_path = arg0;
	printf("\n");
} 

syscall::unlink:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->unlink_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::readlink:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->readlink_path = arg0;
	self->readlink_buf = arg1;
	self->readlink_bufsize = arg2;
	printf("\n");
} 

syscall::readlink:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->readlink_path));
	printf("[%d]; ", self->readlink_bufsize);

	printf("[%s]; ", self->readlink_target); 
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::rename:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->rename_old_path = arg0;
	self->rename_new_path = arg1;
	printf("\n");
} 

syscall::rename:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->rename_old_path));
	printf("[%s]; ", copyinstr(self->rename_new_path));

	printf("[%d]; ", self->rename_inode); 
	printf("[%d]; ", self->rename_type); 
	printf("[%d]; ", self->rename_from_dir_inode); 
	printf("[%d]; ", self->rename_to_dir_inode); 
	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::exchangedata:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->exchangedata_path1 = arg0;
	self->exchangedata_path2 = arg1;
	self->exchangedata_options = arg2;
	printf("\n");
} 

syscall::exchangedata:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->exchangedata_path1));
	printf("[%s]; ", copyinstr(self->exchangedata_path2));
	this->cnode1_ptr = (struct cnode *)(self->exchangedata_vnode1->v_data);
	this->cnode2_ptr = (struct cnode *)(self->exchangedata_vnode2->v_data);
	this->inode1 = (this->cnode1_ptr->c_attr).ca_fileid;
	this->inode2 = (this->cnode2_ptr->c_attr).ca_fileid;
	printf("[%d]; ", this->inode1); 
	printf("[%d]; ", this->inode2); 
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::mkdir:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->mkdir_path = arg0;
	self->mkdir_mode = arg1;
	printf("\n");
} 

syscall::mkdir:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->mkdir_path));
	printf("[%d]; ", self->mkdir_mode);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::flock:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->flock_fd = arg0;
	self->flock_operation = arg1;
	printf("\n");
} 

syscall::flock:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->flock_fd);
	printf("[%d]; ", self->flock_operation);

	printf("[");
	ustack();;
	printf("]; ");
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::mmap:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->mmap_addr = arg0;
	self->mmap_len = arg1;
	self->mmap_prot = arg2;
	self->mmap_flags = arg3;
	self->mmap_fd = arg4;
	self->mmap_offset = arg5;
	printf("\n");
} 

syscall::mmap:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[0x%X]; ", self->mmap_addr);
	printf("[%d]; ", self->mmap_len);
	printf("[%d]; ", self->mmap_prot);
	printf("[%d]; ", self->mmap_flags);
	printf("[%d]; ", self->mmap_fd);
	printf("[%d]; ", self->mmap_offset);
	this->protstr = "";
	this->protstr = (self->mmap_prot & PROT_NONE) ? strjoin(this->protstr, "PROT_NONE ") : this->protstr;
	this->protstr = (self->mmap_prot & PROT_READ) ? strjoin(this->protstr, "PROT_READ ") : this->protstr;
	this->protstr = (self->mmap_prot & PROT_WRITE) ? strjoin(this->protstr, "PROT_WRITE ") : this->protstr;
	this->protstr = (self->mmap_prot & PROT_EXEC) ? strjoin(this->protstr, "PROT_EXEC ") : this->protstr;

	this->flagsstr = "";
	this->flagsstr = (self->mmap_flags & MAP_ANON) ? strjoin(this->flagsstr, "MAP_ANON ") : this->flagsstr;
	this->flagsstr = (self->mmap_flags & MAP_FILE) ? strjoin(this->flagsstr, "MAP_FILE ") : this->flagsstr;
	this->flagsstr = (self->mmap_flags & MAP_FIXED) ? strjoin(this->flagsstr, "MAP_FIXED ") : this->flagsstr;
	this->flagsstr = (self->mmap_flags & MAP_HASSEMAPHORE) ? strjoin(this->flagsstr, "MAP_HASSEMAPHORE ") : this->flagsstr;
	this->flagsstr = (self->mmap_flags & MAP_PRIVATE) ? strjoin(this->flagsstr, "MAP_PRIVATE ") : this->flagsstr;
	this->flagsstr = (self->mmap_flags & MAP_SHARED) ? strjoin(this->flagsstr, "MAP_SHARED ") : this->flagsstr;
	this->flagsstr = (self->mmap_flags & MAP_NOCACHE) ? strjoin(this->flagsstr, "MAP_NOCACHE ") : this->flagsstr;
	printf("[%s]; ", this->protstr); 
	printf("[%s]; ", this->flagsstr); 
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::futimes:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->futimes_fildes = arg0;
	printf("\n");
} 

syscall::futimes:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->futimes_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::utimes:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->utimes_fildes = arg0;
	printf("\n");
} 

syscall::utimes:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->utimes_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::statfs:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->statfs_path = arg0;
	printf("\n");
} 

syscall::statfs:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->statfs_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::statfs64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->statfs64_path = arg0;
	printf("\n");
} 

syscall::statfs64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->statfs64_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fstatfs64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fstatfs64_fildes = arg0;
	printf("\n");
} 

syscall::fstatfs64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fstatfs64_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fstat:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fstat_fildes = arg0;
	printf("\n");
} 

syscall::fstat:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fstat_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::lstat:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->lstat_path = arg0;
	printf("\n");
} 

syscall::lstat:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->lstat_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::stat:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->stat_path = arg0;
	printf("\n");
} 

syscall::stat:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->stat_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fstat64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fstat64_fildes = arg0;
	printf("\n");
} 

syscall::fstat64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fstat64_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::lstat64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->lstat64_path = arg0;
	printf("\n");
} 

syscall::lstat64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->lstat64_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::stat64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->stat64_path = arg0;
	printf("\n");
} 

syscall::stat64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->stat64_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::setattrlist:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->setattrlist_path = arg0;
	printf("\n");
} 

syscall::setattrlist:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->setattrlist_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::getattrlist:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->getattrlist_path = arg0;
	printf("\n");
} 

syscall::getattrlist:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->getattrlist_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::getxattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->getxattr_path = arg0;
	printf("\n");
} 

syscall::getxattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->getxattr_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fgetxattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fgetxattr_fd = arg0;
	printf("\n");
} 

syscall::fgetxattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fgetxattr_fd);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::listxattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->listxattr_path = arg0;
	printf("\n");
} 

syscall::listxattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->listxattr_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::flistxattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->flistxattr_fd = arg0;
	printf("\n");
} 

syscall::flistxattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->flistxattr_fd);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::access:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->access_path = arg0;
	self->access_amode = arg1;
	printf("\n");
} 

syscall::access:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->access_path));
	printf("[%d]; ", self->access_amode);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::chown:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->chown_path = arg0;
	self->chown_owner = arg1;
	self->chown_group = arg2;
	printf("\n");
} 

syscall::chown:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->chown_path));
	printf("[%d]; ", self->chown_owner);
	printf("[%d]; ", self->chown_group);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::chmod:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->chmod_path = arg0;
	self->chmod_mode = arg1;
	printf("\n");
} 

syscall::chmod:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->chmod_path));
	printf("[%d]; ", self->chmod_mode);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fsetxattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fsetxattr_fildes = arg0;
	self->fsetxattr_name = arg1;
	printf("\n");
} 

syscall::fsetxattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fsetxattr_fildes);
	printf("[%s]; ", copyinstr(self->fsetxattr_name));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fsgetpath:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fsgetpath_buf = arg0;
	self->fsgetpath_bufsize = arg1;
	self->fsgetpath_fsid = arg2;
	printf("\n");
} 

syscall::fsgetpath:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fsgetpath_fsid);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::getdirentries64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->getdirentries64_fildes = arg0;
	printf("\n");
} 

syscall::getdirentries64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->getdirentries64_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::getdirentriesattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->getdirentriesattr_fildes = arg0;
	printf("\n");
} 

syscall::getdirentriesattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->getdirentriesattr_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::getfsstat64:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	printf("\n");
} 

syscall::getfsstat64:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::getdtablesize:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	printf("\n");
} 

syscall::getdtablesize:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::chdir:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->chdir_path = arg0;
	printf("\n");
} 

syscall::chdir:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->chdir_path));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fchdir:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fchdir_fildes = arg0;
	printf("\n");
} 

syscall::fchdir:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fchdir_fildes);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::kill:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->kill_pid = arg0;
	self->kill_sig = arg1;
	printf("\n");
} 

syscall::kill:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->kill_pid);
	printf("[%d]; ", self->kill_sig);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fchflags:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fchflags_fd = arg0;
	self->fchflags_flags = arg1;
	printf("\n");
} 

syscall::fchflags:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fchflags_fd);
	printf("[%d]; ", self->fchflags_flags);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fchmod:entry,
syscall::fchmod_extended:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fchmod_fildes = arg0;
	self->fchmod_mode = arg1;
	printf("\n");
} 

syscall::fchmod:return,
syscall::fchmod_extended:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fchmod_fildes);
	printf("[%d]; ", self->fchmod_mode);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fchown:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fchown_fildes = arg0;
	self->fchown_owner = arg1;
	self->fchown_group = arg2;
	printf("\n");
} 

syscall::fchown:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fchown_fildes);
	printf("[%d]; ", self->fchown_owner);
	printf("[%d]; ", self->fchown_group);
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::removexattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->removexattr_path = arg0;
	self->removexattr_name = arg1;
	printf("\n");
} 

syscall::removexattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->removexattr_path));
	printf("[%s]; ", copyinstr(self->removexattr_name));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::fremovexattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->fremovexattr_fd = arg0;
	self->fremovexattr_name = arg1;
	printf("\n");
} 

syscall::fremovexattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%d]; ", self->fremovexattr_fd);
	printf("[%s]; ", copyinstr(self->fremovexattr_name));
	printf("[%d]; ", arg0);
	printf("\n");
} 



syscall::setxattr:entry
/ WATCHCOND / 
{ 
	PRINT_GENERIC_ENTRY
	self->setxattr_path = arg0;
	self->setxattr_name = arg1;
	printf("\n");
} 

syscall::setxattr:return
/ WATCHCOND / 
{ 
	PRINT_GENERIC_RETURN
	printf("[%s]; ", copyinstr(self->setxattr_path));
	printf("[%s]; ", copyinstr(self->setxattr_name));
	printf("[%d]; ", arg0);
	printf("\n");
} 



